*
* Clk_crt.asm - Test BUFFALO I/O for the Ep2711E9 Rev. C board
*               (c)2003, EVBplus.com, Written By Wayne Chu
*
*     Function:	Displays a real time clock on the center of the screen.
*		The current time is displayed in the center of the screen, 
*		and updated every second in real time, becaus ethe time is
*		synchronized by the host pc.
*
*		When a BUFFALO output routine is called by the user program,
*		the PC CRT will switch to a new output screen to display
*		the characters. It will	restore the debugger screen when the
*		program is stopped.
*		After the program is stopped by the esc key, it must be
*		started from the beginning in order to show the new 
*		output screen. 
*
* 		All EVB/EVBU BUFFALO I/O routines are located at $0FA0-$0FCF
*		in stead of $FFA0-$FFCF.
*
*
		org	$40
time:		rmb	12	; enough space to hold time, hh:mm:ss pm
prev_sec:	rmb	1	; previous data in second to be compared with 
*				; the current data in second

STACK:  	equ     $FF
*
        	org     $0800	; EVBplus2 board I/O routines
rs485_recv:     rmb     3	; enables rs485 recv mode
rs485_xmit:     rmb     3	; enables rs485 xmit mode
get_date:       rmb     3	; gets current date from PC
get_time:       rmb     3	; gets current time from PC
outstrg00:	rmb	3	; outputs a string terminated by 0
lcd_ini:	rmb	3	; initializes the 16x2 LCD module
lcd_line1:	rmb	3	; displays 16 char on the first line
lcd_line2:	rmb	3	; displays 16 char on the second line
sel_inst:	rmb	3	; selects instruction before writing LCD module 
sel_data:	rmb	3	; selects data before writing the LCD module
wrt_pulse:	rmb	3	; generates a write pulse to the LCD module
seven_segment:  rmb	3	; convert Accu A to segment pattern, bit 7= DP
*
*
                org     $0FA0	; BUFFALO I/O routines are duplicated here
upcase:         rmb     3
wchek:          rmb     3
dchek:          rmb     3
init_sci:       rmb     3
input:          rmb     3
output:         rmb     3
outlhlf:        rmb     3
outrhlf:        rmb     3
outa:           rmb     3
out1byt:        rmb     3
out1bsp:        rmb     3
out2bsp:        rmb     3
outcrlf:        rmb     3
outstrg:        rmb     3
outstrg0:	rmb	3
inchar:         rmb     3
*

	org	$F000
start:  lds	#STACK
	cli		
*
	ldab	#11		; cursor goes down 11 lines
more_lf:
	jsr	outcrlf
	decb
	bne	more_lf
*
        jsr     space44         ; output 44 space characters
*
* cursor is near the center of the screen
*
back:	ldx	#time
	jsr	get_time	; get the 11 character time string 
*                               ; from pc's clock and it will be stored 
*				; at the ram locations (time to time+10) 
	jsr	update_time	; check if display needs to be updated
*
	beq	back		; time is same from lst check, no update
        jsr     bs11            ; do backspace 11 positions
	ldx	#time		; update display, because it received new
	jsr	outstrg0	; time data from pc
	jmp	back

*
bs11:	ldab	#11
more_bs:
	ldaa	#8
	jsr	output
	decb
	bne	more_bs
	rts
*
space44:
	ldab	#44		; generate 44 spaces
more_sp:
	ldaa	#' '
	jsr	output
	decb
	bne	more_sp
	rts
*
* Previous data in second to be compared with the current data in second,
* if the data are same, no display update occurs.  If new time data is
* received from pc, it will update the clock display. It is very important
* to update the display as seldom as possible. Fast and continuously
* updating may cause system crash, then you have to reset the board.
*
* time data format hh:mm:ss a or hh:mm:ss p

update_time:
	ldaa	7,x		; 6,x= ten second, 7,x = unit second
	cmpa	prev_sec	; 
	bne	copy
	rts			; z=1, time is still same from last update
copy: 	staa	prev_sec	; z=0, accu a = '0' to '9' z flag=0
	rts
*
*	
	org	$FFFE
	fdb	start
	end
*	



