*
* Lcd2.asm ---- 16X2 LCD sample program for the Ep2711E9 Rev. C board
*               (c)2003, EVBplus.com, Written By Wayne Chu
*
*     Function:	This program displays a message on a 16X2 LCD display module
*		by calling sel_inst(instruction), sel_data and wrt_pulse
*		subroutines. These subroutines allow user program to 
*		control a different size of LCD display.
*
*
        	org     $0800	; EVBplus2 board I/O routines
rs485_recv:     rmb     3	; enables rs485 recv mode
rs485_xmit:     rmb     3	; enables rs485 xmit mode
get_date:       rmb     3	; gets date info. from host PC
get_time:       rmb     3	; gets time info. from host PC
outstrg00:	rmb	3	; print a string ended by 0
lcd_ini:	rmb	3	; initializes the 16x2 LCD module
lcd_line1:	rmb	3	; displays 16 char on the first line
lcd_line2:	rmb	3	; displays 16 char on the second line
sel_inst:	rmb	3	; selects instruction before writing LCD module 
sel_data:	rmb	3	; selects data before writing the LCD module
wrt_pulse:	rmb	3	; generates a write pulse to the LCD module
seven_segment:  rmb	3	; convert Accu A to segment pattern, bit 7= DP
*
* Segment conversion table in the monitor firmware:
*
* Binary number:	 	0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F
* Converted to segment code:  	0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F
*
* Binary number:		$10,$11,$12,$13,$14,$15,$16,$17
* Converted to segment code:     G   H   h   J   L   n   o   o
*
* Binary number:  		$18,$19,$1A,$1B,$1C,$1D,$1E,$1F,$20
* Converted to segment code:     P   r   t   U   u   y   _  --  Blank
*
*
                org     $0FA0	; Buffalo I/O routines are duplicated here

upcase:         rmb     3
wchek:          rmb     3
dchek:          rmb     3
init_sci:       rmb     3
input:          rmb     3
output:         rmb     3
outlhlf:        rmb     3
outrhlf:        rmb     3
outa:           rmb     3
out1byt:        rmb     3
out1bsp:        rmb     3
out2bsp:        rmb     3
outcrlf:        rmb     3
outstrg:        rmb     3
outstrg0:	rmb	3
inchar:         rmb     3
*

TEN_MS:		equ	2500	; 2500 x 8 cycles = 20,000 cycles = 10ms
*
STACK:	equ	$FF
*
	org	$F000
	jmp	start

*
delay_10ms:
	pshx
	ldx     #TEN_MS		; 2500 x 8 = 20,000 cycles = 10ms 
del1:	dex			; 3 cycles
	nop			; 2 cycle
	bne	del1		; 3 cycles
	pulx
       	rts
*
start:
	lds	#STACK
	jsr	delay_10ms
	jsr	delay_10ms		; delay 20ms during power on
	cli
*
    	jsr	lcd_ini			; initialize the LCD 
back:	jsr	line1
	jsr	line2
	jsr	delay_10ms
	jsr	delay_10ms
	jmp	back

line1:	jsr    	sel_inst		; select instruction
       	ldaa   	#$80     		; set starting address for the line1
       	jsr    	wrt_pulse
*
       	jsr    	sel_data		; select data
	ldx    	#MSG1			; MSG1 for line1, x points to MSG1
        ldab    #16                     ; send out 16 characters
	jsr	msg_out
	rts	

line2:	jsr    	sel_inst
       	ldaa   	#$C0     		; set starting address for the line2
       	jsr    	wrt_pulse
*
       	jsr    	sel_data
	ldx    	#MSG2			; MSG2 for line2, x points to MSG2
        ldab    #16                     ; send out 16 characters
	jsr	msg_out
	rts	

* at entry, x must point to the begining of the message,
*           b = number of the character to be sent out
*           
msg_out:
	ldaa   	0,x
       	jsr    	wrt_pulse
       	inx
       	decb
       	bne    	msg_out
	rts

*
MSG1:   FCC     "REGULAR/ BIG MAC"
MSG2:   FCC     "CHICKEN MCNUGGET"

*
	org	$FFFE
	fdb	start
	end



